package detector;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

/**
 * @author Roman Tekhov
 */
class ConfigurationReader {
	
	private static final String GEOIP_DB_FILE = "geoip.db.file";
	private static final String OUTPUT_FILE = "output.file";
	private static final String MIN_OUTPUT_SPEED = "min.output.speed";
	private static final String FORMATS_FILE = "formats.file";
	private static final String FORMAT_PREFIX = "format.";
	
	
	static Configuration read(String confFile) {
		String geoIpDb = null;
		String outputFile = null;
		String formatsFile = null;
		Double minOutputSpeed = null;
		
		Map<String, String> logFilesByFormatName = new HashMap<String, String>();
				
		BufferedReader reader = null;
		
		try {
			reader = new BufferedReader(new FileReader(confFile));
			
			String line;
			while((line = reader.readLine()) != null) {
				if(line.length() == 0 || line.charAt(0) == '#') {
					continue;
				}
				
				int i = line.indexOf("=");
				
				if(i > 0 && i != line.length() - 1) {
					String name = line.substring(0, i);
					String value = line.substring(i + 1);
						
					if(GEOIP_DB_FILE.equals(name)) {
						geoIpDb = value;
					} else if(OUTPUT_FILE.equals(name)) {
						outputFile = value;
					} else if(FORMATS_FILE.equals(name)) {
						formatsFile = value;
					} else if(MIN_OUTPUT_SPEED.equals(name)) {
						try {
							minOutputSpeed = Double.parseDouble(value);
						} catch (NumberFormatException e) {
							throw new IllegalArgumentException("Minimal output speed is not a valid floating point number");
						}
					} else if(name.startsWith(FORMAT_PREFIX)) {
						String formatName = name.substring(FORMAT_PREFIX.length());
						
						String logFiles = logFilesByFormatName.get(formatName);
						if(logFiles == null) {
							logFiles = value;
						} else {
							logFiles = logFiles + "," + value;
						}
						
						logFilesByFormatName.put(formatName, logFiles);
					}
				}
			}
		} catch (IOException e) {
			throw new IllegalArgumentException("Error reading configuration properties file", e);
		} finally {
			if(reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					throw new IllegalStateException("Error closing configuration properties file", e);
				}
			}
		}
		
		if(geoIpDb == null) {
			throw new IllegalArgumentException("GeoIP database is not specified");
		}
		
		if(outputFile == null) {
			throw new IllegalArgumentException("Output file location is not specified");
		}
		
		if(formatsFile == null) {
			throw new IllegalArgumentException("Format information containing file location is not specified");
		}
		
		if(logFilesByFormatName.isEmpty()) {
			throw new IllegalArgumentException("No log files specified");
		}
		
		List<LogFormat> formats = FormatReader.read(formatsFile);
		Map<LogFormat, List<String>> filesMap = processFiles(formats, logFilesByFormatName);
		
		return new Configuration(geoIpDb, outputFile, minOutputSpeed, filesMap);
	}


	private static Map<LogFormat, List<String>> processFiles(List<LogFormat> formats,
	                                                         Map<String, String> logFilesByFormatName) {
		
		Map<LogFormat, List<String>> filesMap = new HashMap<LogFormat, List<String>>(logFilesByFormatName.size());
		
		for(Map.Entry<String, String> entry : logFilesByFormatName.entrySet()) {
			String formatName = entry.getKey();
			
			LogFormat formatToUse = null;
			
			for(LogFormat format : formats) {
				if(format.getName().equalsIgnoreCase(formatName)) {
					formatToUse = format;
					break;
				}
			}
			
			if(formatToUse == null) {
				throw new IllegalArgumentException("Unknown format " + formatName);
			}
									
			StringTokenizer st = new StringTokenizer(entry.getValue(), ", ");
			
			List<String> files = filesMap.get(formatToUse);
			if(files == null) {
				files = new ArrayList<String>();
				filesMap.put(formatToUse, files);
			}
			
			while(st.hasMoreTokens()) {
				files.add(st.nextToken());
			}
		}		
		
		return filesMap;
	}
	
}
