package detector;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Roman Tekhov
 */
class FormatReader {
	
	private static final XPath XPATH = XPathFactory.newInstance().newXPath();
	
	private static final XPathExpression FORMATS_EXPR;
	private static final XPathExpression NAME_EXPR;
	private static final XPathExpression REG_EXP_EXPR;
	private static final XPathExpression DELIMITER_EXPR;
	private static final XPathExpression DATE_FORMAT_EXPR;
	private static final XPathExpression DATE_LOCALE_EXPR;
	private static final XPathExpression PART_NUMBER_EXPR;
	private static final XPathExpression DATE_PART_EXPR;
	private static final XPathExpression USER_PART_EXPR;
	private static final XPathExpression HOST_PART_EXPR;
	private static final XPathExpression START_FROM_LINE_EXPR;
	private static final XPathExpression STOP_AT_LINE_EXPR;
	private static final XPathExpression IGNORE_EXPR;
	
	static {
		try {
			FORMATS_EXPR = XPATH.compile("/formats/format");
			NAME_EXPR = XPATH.compile("name/text()");
		  REG_EXP_EXPR = XPATH.compile("reg-exp/text()");
		  DELIMITER_EXPR = XPATH.compile("delimiter/text()");
		  DATE_FORMAT_EXPR = XPATH.compile("date-format/text()");
		  DATE_LOCALE_EXPR = XPATH.compile("date-locale/text()");
		  PART_NUMBER_EXPR = XPATH.compile("part-number/text()");
		  DATE_PART_EXPR = XPATH.compile("date-part/text()");
		  USER_PART_EXPR = XPATH.compile("user-part/text()");
		  HOST_PART_EXPR = XPATH.compile("host-part/text()");
		  START_FROM_LINE_EXPR = XPATH.compile("start-from-line/text()");
		  STOP_AT_LINE_EXPR = XPATH.compile("stop-at-line/text()");
		  IGNORE_EXPR = XPATH.compile("ignore/text()");
		} catch (XPathExpressionException e) {
			throw new IllegalStateException(e);
		} 
	}
	
	
	static List<LogFormat> read(String formatsFile) {		
		Document document;
		try {
			document = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(formatsFile);
		} catch (Exception e) {
			throw new IllegalArgumentException("Error reading formats file", e);
		}
		    
    NodeList formatNodes;
		try {
			formatNodes = (NodeList) FORMATS_EXPR.evaluate(document, XPathConstants.NODESET);
		} catch (XPathExpressionException e) {
			throw new IllegalArgumentException("Error parsing the formats element", e);
		}
    if(formatNodes == null || formatNodes.getLength() == 0) {
    	throw new IllegalArgumentException("No formats specified");
		}
    
    return parseFormats(formatNodes);
	}


	private static List<LogFormat> parseFormats(NodeList formatNodes) {
		List<LogFormat> formats = new ArrayList<LogFormat>(formatNodes.getLength());
		
		for(int i = 0; i < formatNodes.getLength(); i++) {
			Node formatNode = formatNodes.item(i);
			
			LogFormat format = new LogFormat();
			
			format.setName(parseMandatoryAttribute(NAME_EXPR, formatNode, "name", i));
			
			processRegExpOrDelimiter(i, formatNode, format);
						
			format.setPartNumber(parseMandatoryNumberAttribute(PART_NUMBER_EXPR, formatNode, "part number", i));
			format.setDatePart(parseMandatoryNumberAttribute(DATE_PART_EXPR, formatNode, "date part", i));
			format.setUserPart(parseMandatoryNumberAttribute(USER_PART_EXPR, formatNode, "user part", i));
			format.setHostPart(parseMandatoryNumberAttribute(HOST_PART_EXPR, formatNode, "host part", i));
			
			format.setDateFormat(parseMandatoryAttribute(DATE_FORMAT_EXPR, formatNode, "date format", i));
			String localeStr = parseMandatoryAttribute(DATE_LOCALE_EXPR, formatNode, "date locale", i);
			format.setDateLocale(new Locale(localeStr));
			
			format.setStartFromLine(parseAttribute(START_FROM_LINE_EXPR, formatNode, "start from line", i));
			format.setStopAtLine(parseAttribute(STOP_AT_LINE_EXPR, formatNode, "stop at line", i));
			
			format.setIgnore(parseAttribute(IGNORE_EXPR, formatNode, "ignore", i));
						
			formats.add(format);
		}
		
		return formats;
	}


	private static void processRegExpOrDelimiter(int i, Node formatNode, LogFormat format) {
		String regExp = parseAttribute(REG_EXP_EXPR, formatNode, "regular expression", i);
		String delimiter = parseAttribute(DELIMITER_EXPR, formatNode, "delimiter", i);
		
		if(regExp != null && delimiter != null) {
			throw new IllegalArgumentException("Invalid format " + i + 
			                                   " - either 'reg-exp' or 'delimiter' must be specified, not both");
		}
		
		if(regExp == null && delimiter == null) {
			throw new IllegalArgumentException("Invalid format " + i + 
			                                   " - neither 'reg-exp' nor 'delimiter' is specified");
		}
		
		format.setRegExp(regExp);
		format.setDelimiter(delimiter);
	}	
	
	
	private static String parseAttribute(XPathExpression expr, Node formatNode, String name, int i) {
		try {
			String value = expr.evaluate(formatNode);
			
			return "".equals(value) ? null : value;
		} catch (XPathExpressionException e) {
			throw new IllegalArgumentException("Invalid format " + i + " - error parsing " + name);
		}
	}
	
	private static String parseMandatoryAttribute(XPathExpression expr, Node formatNode, String name, int i) {
		String value = parseAttribute(expr, formatNode, name, i);
		
		if(value == null) {
			throw new IllegalArgumentException("Invalid format " + i + " - " + name + " not specified");
		}
		
		return value;
	}
	
	private static int parseMandatoryNumberAttribute(XPathExpression expr, Node formatNode, String name, int i) {
		String valueStr = parseMandatoryAttribute(expr, formatNode, name, i);
		
		try {				
			return Integer.parseInt(valueStr);
		} catch (NumberFormatException e) {
			throw new IllegalArgumentException("Invalid format " + i + " - " + name + " not an integer number: " + valueStr);
		}
	}

}
